package org.deephacks.tools4j.config.internal.core.hbase;

/**
 * Utilities for byte manipulation.
 */
public class BytesUtils extends Bytes {

    /**
     * Adds a big-endian 4-byte integer to a sorted array of bytes.
     * 
     * @param arr This byte array is assumed to be sorted array
     *            of signed ints.
     * @param n value to add.
     * @return new array with the added value.
     */
    public static byte[] add(byte[] arr, int n) {
        int index = binarySearch(arr, n);
        byte[] arr2 = new byte[arr.length + 4];
        System.arraycopy(arr, 0, arr2, 0, index);
        System.arraycopy(arr, index, arr2, index + 4, arr.length - index);
        Bytes.setInt(arr2, n, index);
        return arr2;
    }

    /**
     * Search for a big-endian 4-byte integer in a array of bytes.
     * 
     * @param a array of containing only big-endian 4-byte integers. 
     * @param key the value to seach for.
     * @return the index found.
     */
    public static int binarySearch(byte[] a, int key) {
        int low = 0;
        int high = a.length;

        while (low < high) {
            int mid = (low + high) >>> 1;
            if (mid % 4 != 0) {
                if (high == a.length) {
                    mid = low;
                } else {
                    mid = high;
                }
            }
            int midVal = getInt(a, mid);

            if (midVal < key)
                low = mid + 4;
            else if (midVal > key)
                high = mid - 4;
            else
                return mid; // key found
        }
        if (low == a.length) {
            return low;
        }
        return key > getInt(a, low) ? low + 4 : low;

    }
}